
//
// Jupiter.java -- calculate positions of Saturn's moons.
//
// Algorithm adapted from SATSAT.BAS
//  by Dan Bruton, astro@tamu.edu.
//
//	Copyright 2002 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

package Akk.Astro;

import java.awt.*;
import java.util.Date;

public class Saturn
{
    public class SatMoon
    {
        String name;
        double sma;   // Semimajor axis, in km
        //double ecc;   // eccentricity
        double pd;    // Synodic Orbital Period in earth days
        int mag;      // visual magnitude *10
        double u0;    // Angle from inferior geocentric conjuction,
        // measured westward along the orbit at epoch:
        // these are guesstimated from xephem.
        double u;     // Angle from inferior geocentric conjuction
        // measured westward (we will calculate)

        public SatMoon(String n, double s, double p,
                       int m, double uu)
        {
            name = n;
            sma = s;
            pd = p;
            mag = m;
            u0 = uu;
        }
    }

    final static int NUM_MOONS = 7;
    final static double JDE = 2444238.5;   // Epoch Jan 0.0 1980
                                           //  = December 31,1979 0:0:0 UT
    final static double TWOPI = 2. * Math.PI;
    final static double D2R = Math.PI / 180.;
    final static double earthOrbRate = TWOPI / (1.00004 * 365.2422);
    final static double earthEcc = .016718;
    final static double earthSMA = 1.;
    final static double satOrbRate = TWOPI / (29.45771 * 365.2422);
    final static double satEcc = .0556155;
    final static double satSMA = 9.554747;
    final static double LEE = 98.83354 * D2R;    // Longitude of Earth @ Epoch
    final static double LEP = 102.596403 * D2R;  // Long. of Earth Perihelion
    final static double LpE = 165.322242 * D2R;  // Long. of Saturn at Epoch
    final static double LpP = 92.6653974 * D2R;  // Long. of Sat`s Perihelion
    final static double radius = 60330;   // Radius of planet in km
    XYCoord moonCoords[];

    /*
     * Kludge alert!
     * This table needs to be reset every year or so.
     * Reset the U0 values, the Angle from inferior geocentric conjuction
     * measured westward along the orbit at epoch.
     * Pick a time at which you want accuracy to be highest
     * (probably opposition or a month or two later),
     * calculate the Julian time, and estimate the moon positions
     * for that time.
     */
    double JDEmoons = 2452640.5;    // Jan 1.0 2003
    SatMoon satMoons[] =
    {
        //                      Name           SMA     Period mag*10     U0
        //                      ----           ---     ------ ------    ----
        new SatMoon(new String("Mimas"),    185600,   .9425049, 130, 185.*D2R),
        new SatMoon(new String("Enceladus"),238100,  1.3703731, 118, 230.*D2R),
        new SatMoon(new String("Tethys"),   294700,  1.8880926, 103, 160.*D2R),
        new SatMoon(new String("Dione"),    377500,  2.7375218, 102, 132.*D2R),
        new SatMoon(new String("Rhea"),     527200,  4.519163,   98,  97.*D2R),
        new SatMoon(new String("Titan"),   1221600, 15.9669028 , 84, 137.*D2R),
        new SatMoon(new String("Hyperion"),1483000, 21.3174647, 143, 335.*D2R),
        new SatMoon(new String("Iapetus"), 3560100, 79.9208313, 112, 290.*D2R),
    };

    // XXX Kludge: need to put in the real equations for Saturn's rings!
    double Inclination = 26.0 * D2R;         // Inclination of rings

    public Saturn(StarDate _date)
    {
        setDate(_date);
    }

    int getNumMoons()	{ return NUM_MOONS; }

    String getMoonLabel(int whichmoon)
    {
        switch (whichmoon)
        {
          case 0: return "I";
          case 1: return "II";
          case 2: return "III";
          case 3: return "IV";
          case 4: return "V";
          case 5: return "VI";
          case 6: return "VII";
          case 7: return "VIII";
          default: return "?";
        }
    }

    String getMoonInfo(int whichmoon)
    {
        if (whichmoon > NUM_MOONS || whichmoon < 0)
            return "?";
        return satMoons[whichmoon].name;
    }

    void setDate(StarDate date)
    {
        // Calculate the ring incidence and the positions of the moons.

        // Eventually we want to do this using Planet.class,
        // but for now we'll use special-purpose code.
        // Start by calculating the earth's heliocentric coords:
        //HelioCoords earthc = Planet.earth.calcHeliocentric(jd);
        // Calculate position of Saturn:
        //AstroCoords coords = Planet.Saturn.locate(jd, earthc);

        double JD = date.getJulianDate();
        int i;
        double lE, lP, rE, rP, dt, ii, f, f1;
        double psi, lpq, rpq, lamb, beta, RA, dec;
        double tva, pva, tvc, pvc, dot1, tvb, pvb, dot2;
        double trip;
        double IapInci, IapGam;

        /******************* FIND MOON ORBITAL ANGLES ************************/
        double NN = JD - JDE;   /* NN = Number of days since epoch */
        double ME = ((earthOrbRate * NN) + LEE - LEP);  // Mean Anom, Earth
        double MP = ((satOrbRate * NN) + LpE - LpP);    // Mean Anom, Saturn
        double VE = ME;
        double VP = MP;   /* True Anomolies - Solve Kepler's Equation */
        for (i=0; i < 3; ++i)
        {
            VE = VE - (VE - (earthEcc * Math.sin(VE)) - ME)
                / (1 - (earthEcc * Math.cos(VE)));
            VP = VP - (VP - (satEcc * Math.sin(VP)) - MP)
                / (1 - (satEcc * Math.cos(VP)));
        }

        VE = 2. * Math.atan(Math.sqrt((1. + earthEcc)
                                      / (1. - earthEcc)) * Math.tan(VE / 2.));
        if (VE < 0) VE = TWOPI + VE;
        VP = 2 * Math.atan(Math.sqrt((1. + satEcc)
                                     / (1. - satEcc)) * Math.tan(VP / 2.));
        if (VP < 0) VP = TWOPI + VP;

        /* Heliocentric Longitudes of Earth and Saturn */
        lE = VE + LEP;
        if (lE > TWOPI) lE = lE - TWOPI;
        lP = VP + LpP;
        if (lP > TWOPI) lP = lP - TWOPI;

        /* Distances of Earth and Saturn from the Sun in AU's */
        rE = earthSMA * (1. - earthEcc * earthEcc)
            / (1. + earthEcc * Math.cos(VE));
        rP = satSMA * (1. - satEcc * satEcc)
            / (1. + satEcc * Math.cos(VP));
        /* DT = Distance from Saturn to Earth in AU's - Law of Cosines */
        dt = Math.sqrt((rE * rE) + (rP * rP)
                       - (2. * rE * rP * Math.cos(lE - lP)));

        /* II = Angle between Earth and Sun as seen from Saturn */
        ii = Math.asin(rE * Math.sin(lP - lP) / dt);

        /* For the moon computations, we need to compute
         * from JDEmoons rather than JDE.
         */
        /* F = NN - (Light Time to Earth in days) */
        f = JD - JDEmoons - (dt / 173.83);
        f1 = ii + MP - VP;

        // Loop over moons
        for (i=0; i < satMoons.length; ++i)
        {
            satMoons[i].u = ((satMoons[i].u0
                              + (f * TWOPI / satMoons[i].pd) + f1)
                             / TWOPI);
            satMoons[i].u = (satMoons[i].u - (int)satMoons[i].u) * TWOPI;
        }
    }
    
    XYCoord getMoonXY(int whichmoon)
    {
        if (whichmoon >= satMoons.length || whichmoon < 0)
            return new XYCoord(Double.POSITIVE_INFINITY,
                               Double.POSITIVE_INFINITY);
    	return new XYCoord(
            -((satMoons[whichmoon].sma * Math.sin(satMoons[whichmoon].u))
              / radius),
            (satMoons[whichmoon].sma * Math.cos(satMoons[whichmoon].u)
             * Math.sin(Inclination)) / radius
            );
    }
}

